/* eslint-disable no-console */
import type { HighlighterCore, LanguageRegistration, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { createContext, useContext, useEffect, useMemo, useState } from 'react'

const DEBUG = true
function log(...args: any[]) {
  if (DEBUG)
    console.log('[ShikiProvider]', ...args)
}

interface ShikiContextValue {
  highlighter: HighlighterCore | null
  isReady: boolean
  error: Error | null
  getLoadedLanguages: () => string[]
  getLoadedThemes: () => string[]
}

interface ShikiProviderProps {
  children: React.ReactNode
  initialLanguages: LanguageRegistration[]
  initialThemes: ThemeRegistrationAny[]
}

const ShikiContext = createContext<ShikiContextValue | null>(null)

export function ShikiProvider({ children, initialLanguages, initialThemes }: ShikiProviderProps) {
  const [highlighter, setHighlighter] = useState<HighlighterCore | null>(null)
  const [isReady, setIsReady] = useState(false)
  const [error, setError] = useState<Error | null>(null)

  useEffect(() => {
    let mounted = true
    const initHighlighter = async () => {
      try {
        log('Initializing highlighter...')

        const engine = createJavaScriptRegexEngine({
          forgiving: true,
          cache: new Map(),
        })

        const instance = await createHighlighterCore({
          engine,
        })

        // Load themes first
        log('Loading themes...')
        for (const theme of initialThemes) {
          await instance.loadTheme(theme)
        }

        // Then load languages
        log('Loading languages...')
        for (const lang of initialLanguages) {
          await instance.loadLanguage(lang)
        }

        if (mounted) {
          setHighlighter(instance)
          setIsReady(true)
          log('Highlighter ready with:', {
            languages: instance.getLoadedLanguages(),
            themes: instance.getLoadedThemes(),
          })
        }
      }
      catch (err) {
        console.error('Failed to initialize highlighter:', err)
        if (mounted) {
          setError(err instanceof Error ? err : new Error('Failed to initialize highlighter'))
        }
      }
    }

    initHighlighter()
    return () => {
      mounted = false
    }
  }, []) // Intentionally empty deps since initialLanguages/Themes should be stable

  const value = useMemo<ShikiContextValue>(
    () => ({
      highlighter,
      isReady,
      error,
      getLoadedLanguages: () => highlighter?.getLoadedLanguages() || [],
      getLoadedThemes: () => highlighter?.getLoadedThemes() || [],
    }),
    [highlighter, isReady, error],
  )

  return <ShikiContext.Provider value={value}>{children}</ShikiContext.Provider>
}

export function useShiki() {
  const context = useContext(ShikiContext)
  if (!context) {
    throw new Error('useShiki must be used within a ShikiProvider')
  }
  return context
}
