import type { ThemedToken, ThemeRegistration } from '@shikijs/types'
import React, { memo, useCallback, useEffect, useState } from 'react'
import { ScrollView, StyleSheet, Text, View } from 'react-native'
import { useShiki } from './SyntaxContext'

const styles = StyleSheet.create({
  container: {
    flex: 1,
  },
  line: {
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
})

interface SyntaxHighlighterProps {
  code: string
  language: string
  theme?: string | ThemeRegistration
  fontSize?: number
}

export const SyntaxHighlighter = memo(({ code, language, theme = 'github-dark', fontSize = 14 }: SyntaxHighlighterProps) => {
  const { highlighter, isReady } = useShiki()
  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<string | null>(null)

  const highlight = useCallback(async () => {
    if (!highlighter || !isReady)
      return

    try {
      // Get loaded languages for validation
      const loadedLangs = highlighter.getLoadedLanguages()
      const normalizedLang = language.toLowerCase()

      // Use text if language isn't loaded
      const effectiveLang = loadedLangs.includes(normalizedLang) ? normalizedLang : 'text'

      const themeName = typeof theme === 'string' ? theme : theme.name

      const results = highlighter.codeToTokensBase(code, {
        lang: effectiveLang,
        theme: themeName,
      })

      setTokens(results)
      setError(null)
    }
    catch (err) {
      console.error('Highlighting error:', err)
      setError(err instanceof Error ? err.message : 'Failed to highlight code')
    }
  }, [code, language, theme, highlighter, isReady])

  useEffect(() => {
    highlight()
  }, [highlight])

  if (!isReady) {
    return (
      <View style={styles.container}>
        <Text style={{ fontSize }}>{code}</Text>
      </View>
    )
  }

  if (error) {
    return (
      <View style={styles.container}>
        <Text style={{ color: 'red' }}>{error}</Text>
        <Text style={{ fontSize }}>{code}</Text>
      </View>
    )
  }

  return (
    <ScrollView horizontal style={styles.container} contentContainerStyle={{ padding: 10 }}>
      <View>
        {tokens.map((line, i) => (
          <View key={i} style={styles.line}>
            {line.map((token, j) => (
              <Text
                key={`${i}-${j}`}
                style={{
                  color: token.color,
                  fontSize,
                  ...(token.fontStyle && token.fontStyle & 1 && { fontStyle: 'italic' }),
                  ...(token.fontStyle && token.fontStyle & 2 && { fontWeight: 'bold' }),
                }}
              >
                {token.content}
              </Text>
            ))}
          </View>
        ))}
      </View>
    </ScrollView>
  )
})
